{
    "type": "record",
    "name": "AlarmLocation",
    "namespace": "org.jlab.jaws.entity",
    "doc": "Value of a named location",
    "fields": [
        {
            "name": "parent",
            "type": [
              "null",
              {
                "type": "string",
                "avro.java.string": "String"
              }
            ],
            "doc": "The parent location, or null if a top-level location"
        }
    ]
}
