{
    "type": "record",
    "name": "AlarmOverrideUnion",
    "namespace": "org.jlab.jaws.entity",
    "doc": "Wraps the union of suppressed alarm values",
    "fields": [
        {
            "name": "msg",
            "type": [
                "DisabledOverride",
                "FilteredOverride",
                "LatchedOverride",
                "MaskedOverride",
                "OnDelayedOverride",
                "OffDelayedOverride",
                "ShelvedOverride"
            ],
            "doc": "Suppressed alarm value modeled as a nested union to avoid complications of union at root of schema."
        }
    ]
}
