{
  "type": "record",
  "name": "OverrideList",
  "namespace": "org.jlab.jaws.entity",
  "fields": [
    {
      "name": "overrides",
      "type": {
        "type": "array",
        "items": {
          "name": "override",
          "type": "AlarmOverrideUnion"
        }
      },
      "doc": "The alarm overrides"
    }
  ]
}