{
    "type": "record",
    "name": "IntermediateMonolog",
    "namespace": "org.jlab.jaws.entity",
    "fields": [
        {
            "name": "registration",
            "type": "EffectiveRegistration",
            "doc": "The effective AlarmRegistration considering class defaults"
        },
        {
            "name": "activation",
            "type": "EffectiveActivation",
            "doc": "The effective AlarmActivation considering overrides"
        },
        {
            "name": "transitions",
            "type": "ProcessorTransitions",
            "doc": "Transition states used by the alarm processor"
        }
    ]
}
