{
    "type": "record",
    "name": "DisabledOverride",
    "namespace": "org.jlab.jaws.entity",
    "doc": "A disabled override",
    "fields": [
        {
            "name": "comments",
            "type": [
                "null",
                {
                    "type": "string",
                    "avro.java.string": "String"
                }
            ],
            "doc": "Operator explanation"
        }
    ]
}
