{
    "type": "record",
    "name": "FilteredOverride",
    "namespace": "org.jlab.jaws.entity",
    "doc": "A filtered override",
    "fields": [
        {
            "name": "filtername",
            "type": {
                "type": "string",
                "avro.java.string": "String"
            },
            "doc": "The name of the filter suppressing this alarm"
        }
    ]
}
