{
    "type": "record",
    "name": "OnDelayedOverride",
    "namespace": "org.jlab.jaws.entity",
    "doc": "An on-delayed override",
    "fields": [
        {
            "name": "expiration",
            "type": "long",
            "doc": "Unix timestamp of milliseconds since Epoch of Jan 1. 1970 - Timestamp denotes when this delayed alarm should expire"
        }
    ]
}
