"""
   Code to facilitate Event Sourcing.

   See Also:
       - `Storing Data in Kafka <https://www.confluent.io/blog/okay-store-data-apache-kafka/>`_
       - `Fowler on Event Sourcing <ttps://martinfowler.com/eaaDev/EventSourcing.html>`_
"""