{
    "type": "record",
    "name": "ShelvedOverride",
    "namespace": "org.jlab.jaws.entity",
    "doc": "A temporarily suppressed alarm override",
    "fields": [
        {
            "name": "oneshot",
            "type": "boolean",
            "doc": "Indicates whether this shelved alarm unshelves when active; otherwise considered a continuous shelve and clears only upon expiration or manual operator intervention",
            "default": false
        },
        {
            "name": "expiration",
            "type": "long",
            "doc": "Unix timestamp of milliseconds since Epoch of Jan 1. 1970 - Timestamp denotes when this shelved alarm should expire"
        },
        {
            "name": "reason",
            "type": {
                "type": "enum",
                "name": "ShelvedReason",
                "doc": "Enumeration of possible reasons",
                "symbols": [
                    "Stale_Alarm",
                    "Chattering_Fleeting_Alarm",
                    "Other"
                ]
            },
            "doc": "Reason this alarm is shelved"
        },
        {
            "name": "comments",
            "type": [
                "null",
                {
                    "type": "string",
                    "avro.java.string": "String"
                }
            ],
            "doc": "Operator explanation"
        }
    ]
}
